@@ -1,6 +1,6 @@
 // dwp.h -- general definitions for dwp.
 
-// Copyright (C) 2012-2021 Free Software Foundation, Inc.
+// Copyright (C) 2012-2024 Free Software Foundation, Inc.
 // Written by Cary Coutant <ccoutant@google.com>.
 
 // This file is part of dwp, the DWARF packaging utility.
@@ -94,7 +94,7 @@ gold_info(const char* msg, ...) ATTRIBUTE_PRINTF_1;
 
 #define gold_unreachable() \
   (gold::do_gold_unreachable(__FILE__, __LINE__, \
-			     static_cast<const char*>(__FUNCTION__)))
+			     static_cast<const char*>(__func__)))
 
 extern void do_gold_unreachable(const char*, int, const char*)
   ATTRIBUTE_NORETURN;
