@@ -26,12 +26,6 @@
 #include "tsd.h"
 #include "memmgr.h"
 
-/* TprintfT(<level>,...) definitions.  Adjust per module as needed */
-#define DBG_LT0 0 // for high-level configuration, unexpected errors/warnings
-#define DBG_LT1 1 // for configuration details, warnings
-#define DBG_LT2 2
-#define DBG_LT3 3
-
 /*
  * Build our thread-specific-data support on pthread interfaces.
  */
