@@ -1,4 +1,4 @@
-/* Copyright (C) 2021-2023 Free Software Foundation, Inc.
+/* Copyright (C) 2021-2024 Free Software Foundation, Inc.
    Contributed by Oracle.
 
    This file is part of GNU Binutils.
@@ -32,6 +32,10 @@
 #include "i18n.h"
 #include "debug.h"
 
+#ifndef O_LARGEFILE
+#define O_LARGEFILE 0
+#endif
+
 #define SWAP_ENDIAN(x)  swapByteOrder((void *) (&(x)), sizeof(x))
 #define AppendString(len, arr, ...) len += snprintf(arr + len, sizeof(arr) - len, __VA_ARGS__)
 #define ARR_SIZE(x)     (sizeof (x) / sizeof (*(x)))
