@@ -1,6 +1,6 @@
 #!/bin/sh
 # genscripts.sh - generate the ld-emulation-target specific files
-# Copyright (C) 2004-2023 Free Software Foundation, Inc.
+# Copyright (C) 2004-2024 Free Software Foundation, Inc.
 #
 # This file is part of the Gnu Linker.
 #
@@ -64,35 +64,50 @@
 # following suffixes might be generated as well:
 #
 # xdwe:   -pie    -z combreloc -z separate-code -z relro -z now
+# xdwer:  -pie    -z combreloc -z separate-code -z relro -z now -z one-rosegment
 # xdw:    -pie    -z combreloc                  -z relro -z now
 # xdceo:  -pie    -z combreloc -z separate-code -z relro
+# xdceor: -pie    -z combreloc -z separate-code -z relro        -z one-rosegment
 # xdce:   -pie    -z combreloc -z separate-code
+# xdcer:  -pie    -z combreloc -z separate-code                 -z one-rosegment
 # xdco:   -pie    -z combreloc                  -z relro
 # xdc:    -pie    -z combreloc
 # xdeo:   -pie                 -z separate-code -z relro
+# xdeor:  -pie                 -z separate-code -z relro        -z one-rosegment
 # xde:    -pie                 -z separate-code
+# xder:   -pie                 -z separate-code                 -z one-rosegment
 # xdo:    -pie                                  -z relro
 # xd:     -pie
 #
 # xswe:   -shared -z combreloc -z separate-code -z relro -z now
+# xswer:  -shared -z combreloc -z separate-code -z relro -z now -z one-rosegment
 # xsw:    -shared -z combreloc                  -z relro -z now
 # xsceo:  -shared -z combreloc -z separate-code -z relro
+# xsceor: -shared -z combreloc -z separate-code -z relro        -z one-rosegment
 # xsce:   -shared -z combreloc -z separate-code
+# xscer:  -shared -z combreloc -z separate-code                 -z one-rosegment
 # xsco:   -shared -z combreloc                  -z relro
 # xsc:    -shared -z combreloc
 # xseo:   -shared              -z separate-code -z relro
+# xseor:  -shared              -z separate-code -z relro        -z one-rosegment
 # xse:    -shared              -z separate-code
+# xser:   -shared              -z separate-code                 -z one-rosegment
 # xso:    -shared                               -z relro
 # xs:     -shared
 #
-# xwe:            -z combreloc -z separate-code -z relro -z now
+# xwe:            -z combreloc -z separate-code -z relro -z now -z one-rosegment
+# xwer:           -z combreloc -z separate-code -z relro -z now
 # xw:             -z combreloc                  -z relro -z now
 # xceo:           -z combreloc -z separate-code -z relro
+# xceor:          -z combreloc -z separate-code -z relro        -z one-rosegment
 # xce:            -z combreloc -z separate-code
+# xcer:           -z combreloc -z separate-code                 -z one-rosegment
 # xco:            -z combreloc                  -z relro
 # xc:             -z combreloc
 # xeo:                         -z separate-code -z relro
+# xeor:                        -z separate-code -z relro        -z one-rosegment
 # xe:                          -z separate-code
+# xer:                         -z separate-code                 -z one-rosegment
 # xo:                                           -z relro
 #
 #
@@ -143,12 +158,6 @@ fi
 CUSTOMIZER_SCRIPT="${srcdir}/emulparams/${EMULATION_NAME}.sh"
 source_sh ${CUSTOMIZER_SCRIPT}
 
-if test -d ldscripts; then
-  true
-else
-  mkdir ldscripts
-fi
-
 # Set some flags for the emultempl scripts.  USE_LIBPATH will
 # be set for any libpath-using emulation; NATIVE will be set for a
 # libpath-using emulation where ${host} = ${target}.  NATIVE
@@ -301,6 +310,8 @@ DATA_ALIGNMENT_N="${DATA_ALIGNMENT_N-${DATA_ALIGNMENT-.}}"
 DATA_ALIGNMENT_r="${DATA_ALIGNMENT_r-${DATA_ALIGNMENT-}}"
 DATA_ALIGNMENT_u="${DATA_ALIGNMENT_u-${DATA_ALIGNMENT_r}}"
 
+# Create scripts using different settings of the LD_FLAG variable
+
 LD_FLAG=r
 DATA_ALIGNMENT=${DATA_ALIGNMENT_r}
 DEFAULT_DATA_ALIGNMENT="ALIGN(${SEGMENT_SIZE})"
@@ -331,20 +342,36 @@ LD_FLAG=textonly
   source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
 ) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xe
 
+LD_FLAG=rotextonly
+( echo "/* Script for -z separate-code -z one-rosegment */"
+  source_sh ${CUSTOMIZER_SCRIPT}
+  source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
+) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xer
+
 if test -n "$GENERATE_RELRO_SCRIPT"; then
-    LD_FLAG=
     RELRO=" "
+
+    LD_FLAG=
     ( echo "/* Script for -z relro */"
       source_sh ${CUSTOMIZER_SCRIPT}
       source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
     ) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xo
+
     LD_FLAG=textonly
     ( echo "/* Script for -z separate-code -z relro */"
       source_sh ${CUSTOMIZER_SCRIPT}
       source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
     ) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xeo
+
+    LD_FLAG=rotextonly
+    ( echo "/* Script for -z separate-code -z relro -z one-rosegment */"
+      source_sh ${CUSTOMIZER_SCRIPT}
+      source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
+    ) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xeor
+
     unset RELRO
 fi
+
 LD_FLAG=n
 DATA_ALIGNMENT=${DATA_ALIGNMENT_n}
 ( echo "/* Script for -n */"
@@ -368,6 +395,7 @@ if test -n "$GENERATE_COMBRELOC_SCRIPT"; then
     source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
   ) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xc
   rm -f ${COMBRELOC}
+  
   LD_FLAG=ctextonly
   COMBRELOC=ldscripts/${EMULATION_NAME}.xce.tmp
   ( echo "/* Script for -z combreloc -z separate-code */"
@@ -375,7 +403,17 @@ if test -n "$GENERATE_COMBRELOC_SCRIPT"; then
     source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
   ) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xce
   rm -f ${COMBRELOC}
+
+  LD_FLAG=roctextonly
+  COMBRELOC=ldscripts/${EMULATION_NAME}.xcer.tmp
+  ( echo "/* Script for -z combreloc -z separate-code -z one-rosegment */"
+    source_sh ${CUSTOMIZER_SCRIPT}
+    source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
+  ) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xcer
+  rm -f ${COMBRELOC}
+
   RELRO_NOW=" "
+
   LD_FLAG=w
   COMBRELOC=ldscripts/${EMULATION_NAME}.xw.tmp
   ( echo "/* Script for -z combreloc -z relro -z now */"
@@ -383,6 +421,7 @@ if test -n "$GENERATE_COMBRELOC_SCRIPT"; then
     source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
   ) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xw
   rm -f ${COMBRELOC}
+
   LD_FLAG=wtextonly
   COMBRELOC=ldscripts/${EMULATION_NAME}.xwe.tmp
   ( echo "/* Script for -z combreloc -z separate-code -z relro -z now */"
@@ -391,16 +430,29 @@ if test -n "$GENERATE_COMBRELOC_SCRIPT"; then
   ) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xwe
   rm -f ${COMBRELOC}
   COMBRELOC=
+
+  LD_FLAG=rowtextonly
+  COMBRELOC=ldscripts/${EMULATION_NAME}.xwer.tmp
+  ( echo "/* Script for -z combreloc -z separate-code -z relro -z now -z one-rosegment */"
+    source_sh ${CUSTOMIZER_SCRIPT}
+    source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
+  ) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xwer
+  rm -f ${COMBRELOC}
+  COMBRELOC=
+
   unset RELRO_NOW
+
   if test -n "$GENERATE_RELRO_SCRIPT"; then
-      LD_FLAG=c
       RELRO=" "
+
+      LD_FLAG=c
       COMBRELOC=ldscripts/${EMULATION_NAME}.xco.tmp
       ( echo "/* Script for -z combreloc -z relro */"
 	source_sh ${CUSTOMIZER_SCRIPT}
 	source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
       ) | sed -e '/^ *$/d;s/[    ]*$//' > ldscripts/${EMULATION_NAME}.xco
       rm -f ${COMBRELOC}
+
       LD_FLAG=ctextonly
       COMBRELOC=ldscripts/${EMULATION_NAME}.xceo.tmp
       ( echo "/* Script for -z combreloc -z separate-code -z relro */"
@@ -409,38 +461,66 @@ if test -n "$GENERATE_COMBRELOC_SCRIPT"; then
       ) | sed -e '/^ *$/d;s/[    ]*$//' > ldscripts/${EMULATION_NAME}.xceo
       rm -f ${COMBRELOC}
       COMBRELOC=
+
+      LD_FLAG=roctextonly
+      COMBRELOC=ldscripts/${EMULATION_NAME}.xceor.tmp
+      ( echo "/* Script for -z combreloc -z separate-code -z relro -z one-rosegment */"
+	source_sh ${CUSTOMIZER_SCRIPT}
+	source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
+      ) | sed -e '/^ *$/d;s/[    ]*$//' > ldscripts/${EMULATION_NAME}.xceor
+      rm -f ${COMBRELOC}
+      COMBRELOC=
+
       unset RELRO
   fi
 fi
 
 if test -n "$GENERATE_SHLIB_SCRIPT"; then
-  DATA_ALIGNMENT=${DATA_ALIGNMENT_s-${DATA_ALIGNMENT_}}
   CREATE_SHLIB=" "
+  DATA_ALIGNMENT=${DATA_ALIGNMENT_s-${DATA_ALIGNMENT_}}
+
   LD_FLAG=shared
   ( echo "/* Script for -shared */"
     source_sh ${CUSTOMIZER_SCRIPT}
     source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
   ) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xs
+
   LD_FLAG=sharedtextonly
   ( echo "/* Script for -shared -z separate-code */"
     source_sh ${CUSTOMIZER_SCRIPT}
     source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
   ) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xse
 
+  LD_FLAG=rosharedtextonly
+  ( echo "/* Script for -shared -z separate-code -z one-rosegment */"
+    source_sh ${CUSTOMIZER_SCRIPT}
+    source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
+  ) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xser
+
   if test -n "$GENERATE_RELRO_SCRIPT"; then
       RELRO=" "
+
       LD_FLAG=shared
       ( echo "/* Script for -shared -z relro */"
 	source_sh ${CUSTOMIZER_SCRIPT}
 	source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
       ) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xso
+
       LD_FLAG=sharedtextonly
       ( echo "/* Script for -shared -z separate-code -z relro */"
 	source_sh ${CUSTOMIZER_SCRIPT}
 	source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
       ) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xseo
+
+      LD_FLAG=rosharedtextonly
+      ( echo "/* Script for -shared -z separate-code -z relro -z one-rosegment */"
+	source_sh ${CUSTOMIZER_SCRIPT}
+	source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
+      ) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xseor
+
       unset RELRO
   fi
+
   if test -n "$GENERATE_COMBRELOC_SCRIPT"; then
     DATA_ALIGNMENT=${DATA_ALIGNMENT_sc-${DATA_ALIGNMENT}}
     LD_FLAG=cshared
@@ -450,6 +530,7 @@ if test -n "$GENERATE_SHLIB_SCRIPT"; then
       source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
     ) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xsc
     rm -f ${COMBRELOC}
+
     LD_FLAG=csharedtextonly
     COMBRELOC=ldscripts/${EMULATION_NAME}.xsce.tmp
     ( echo "/* Script for -shared -z combreloc -z separate-code */"
@@ -457,7 +538,17 @@ if test -n "$GENERATE_SHLIB_SCRIPT"; then
       source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
     ) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xsce
     rm -f ${COMBRELOC}
+
+    LD_FLAG=rocsharedtextonly
+    COMBRELOC=ldscripts/${EMULATION_NAME}.xscer.tmp
+    ( echo "/* Script for -shared -z combreloc -z separate-code -z one-rosegment */"
+      source_sh ${CUSTOMIZER_SCRIPT}
+      source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
+    ) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xscer
+    rm -f ${COMBRELOC}
+
     RELRO_NOW=" "
+
     LD_FLAG=wshared
     COMBRELOC=ldscripts/${EMULATION_NAME}.xsw.tmp
     ( echo "/* Script for -shared -z combreloc -z relro -z now */"
@@ -465,6 +556,7 @@ if test -n "$GENERATE_SHLIB_SCRIPT"; then
       source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
     ) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xsw
     rm -f ${COMBRELOC}
+
     LD_FLAG=wsharedtextonly
     COMBRELOC=ldscripts/${EMULATION_NAME}.xswe.tmp
     ( echo "/* Script for -shared -z combreloc -z separate-code -z relro -z now */"
@@ -472,17 +564,28 @@ if test -n "$GENERATE_SHLIB_SCRIPT"; then
       source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
     ) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xswe
     rm -f ${COMBRELOC}
+
+    LD_FLAG=rowsharedtextonly
+    COMBRELOC=ldscripts/${EMULATION_NAME}.xswe.tmp
+    ( echo "/* Script for -shared -z combreloc -z separate-code -z relro -z now -z one-rosegment */"
+      source_sh ${CUSTOMIZER_SCRIPT}
+      source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
+    ) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xswer
+    rm -f ${COMBRELOC}
+
     unset RELRO_NOW
 
     if test -n "$GENERATE_RELRO_SCRIPT"; then
-	LD_FLAG=wshared
 	RELRO=" "
+
+	LD_FLAG=wshared
 	COMBRELOC=ldscripts/${EMULATION_NAME}.xsco.tmp
 	( echo "/* Script for -shared -z combreloc -z relro */"
 	  source_sh ${CUSTOMIZER_SCRIPT}
 	  source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
 	) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xsco
 	rm -f ${COMBRELOC}
+
 	LD_FLAG=wsharedtextonly
 	COMBRELOC=ldscripts/${EMULATION_NAME}.xsceo.tmp
 	( echo "/* Script for -shared -z combreloc -z separate-code -z relro */"
@@ -490,40 +593,70 @@ if test -n "$GENERATE_SHLIB_SCRIPT"; then
 	  source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
 	) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xsceo
 	rm -f ${COMBRELOC}
+
+	LD_FLAG=rowsharedtextonly
+	COMBRELOC=ldscripts/${EMULATION_NAME}.xsceor.tmp
+	( echo "/* Script for -shared -z combreloc -z separate-code -z relro -z one-rosegment */"
+	  source_sh ${CUSTOMIZER_SCRIPT}
+	  source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
+	) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xsceor
+	rm -f ${COMBRELOC}
+
 	unset RELRO
     fi
+
     COMBRELOC=
   fi
+
   unset CREATE_SHLIB
 fi
 
 if test -n "$GENERATE_PIE_SCRIPT"; then
-  DATA_ALIGNMENT=${DATA_ALIGNMENT_s-${DATA_ALIGNMENT_}}
   CREATE_PIE=" "
+
+  DATA_ALIGNMENT=${DATA_ALIGNMENT_s-${DATA_ALIGNMENT_}}
   LD_FLAG=pie
   ( echo "/* Script for -pie */"
     source_sh ${CUSTOMIZER_SCRIPT}
     source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
   ) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xd
+
   LD_FLAG=pietextonly
   ( echo "/* Script for -pie -z separate-code */"
     source_sh ${CUSTOMIZER_SCRIPT}
     source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
   ) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xde
+
+  LD_FLAG=ropietextonly
+  ( echo "/* Script for -pie -z separate-code -z one-rosegment */"
+    source_sh ${CUSTOMIZER_SCRIPT}
+    source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
+  ) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xder
+
   if test -n "$GENERATE_RELRO_SCRIPT"; then
       RELRO=" "
+
       LD_FLAG=pie
       ( echo "/* Script for -pie -z relro */"
 	source_sh ${CUSTOMIZER_SCRIPT}
 	source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
       ) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xdo
+
       LD_FLAG=pietextonly
       ( echo "/* Script for -pie -z separate-code -z relro */"
 	source_sh ${CUSTOMIZER_SCRIPT}
 	source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
       ) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xdeo
+
+      LD_FLAG=ropietextonly
+      ( echo "/* Script for -pie -z separate-code -z relro -z one-rosegment */"
+	source_sh ${CUSTOMIZER_SCRIPT}
+	source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
+      ) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xdeor
+
       unset RELRO
   fi
+
   if test -n "$GENERATE_COMBRELOC_SCRIPT"; then
     DATA_ALIGNMENT=${DATA_ALIGNMENT_sc-${DATA_ALIGNMENT}}
     COMBRELOC=ldscripts/${EMULATION_NAME}.xdc.tmp
@@ -533,6 +666,7 @@ if test -n "$GENERATE_PIE_SCRIPT"; then
       source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
     ) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xdc
     rm -f ${COMBRELOC}
+
     LD_FLAG=cpietextonly
     COMBRELOC=ldscripts/${EMULATION_NAME}.xdce.tmp
     ( echo "/* Script for -pie -z combreloc -z separate-code */"
@@ -540,7 +674,17 @@ if test -n "$GENERATE_PIE_SCRIPT"; then
       source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
     ) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xdce
     rm -f ${COMBRELOC}
+
+    LD_FLAG=rocpietextonly
+    COMBRELOC=ldscripts/${EMULATION_NAME}.xdcer.tmp
+    ( echo "/* Script for -pie -z combreloc -z separate-code -z one-rosegment */"
+      source_sh ${CUSTOMIZER_SCRIPT}
+      source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
+    ) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xdcer
+    rm -f ${COMBRELOC}
+
     RELRO_NOW=" "
+
     LD_FLAG=wpie
     COMBRELOC=ldscripts/${EMULATION_NAME}.xdw.tmp
     ( echo "/* Script for -pie -z combreloc -z relro -z now */"
@@ -548,6 +692,7 @@ if test -n "$GENERATE_PIE_SCRIPT"; then
       source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
     ) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xdw
     rm -f ${COMBRELOC}
+
     LD_FLAG=wpietextonly
     COMBRELOC=ldscripts/${EMULATION_NAME}.xdwe.tmp
     ( echo "/* Script for -pie -z combreloc -z separate-code -z relro -z now */"
@@ -555,17 +700,28 @@ if test -n "$GENERATE_PIE_SCRIPT"; then
       source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
     ) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xdwe
     rm -f ${COMBRELOC}
+
+    LD_FLAG=rowpietextonly
+    COMBRELOC=ldscripts/${EMULATION_NAME}.xdwer.tmp
+    ( echo "/* Script for -pie -z combreloc -z separate-code -z relro -z now -z one-rosegment */"
+      source_sh ${CUSTOMIZER_SCRIPT}
+      source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
+    ) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xdwer
+    rm -f ${COMBRELOC}
+
     unset RELRO_NOW
 
     if test -n "$GENERATE_RELRO_SCRIPT"; then
-	LD_FLAG=wpie
 	RELRO=" "
+
+	LD_FLAG=wpie
 	COMBRELOC=ldscripts/${EMULATION_NAME}.xdco.tmp
 	( echo "/* Script for -pie -z combreloc -z relro */"
 	  source_sh ${CUSTOMIZER_SCRIPT}
 	  source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
 	) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xdco
 	rm -f ${COMBRELOC}
+
 	LD_FLAG=wpietextonly
 	COMBRELOC=ldscripts/${EMULATION_NAME}.xdceo.tmp
 	( echo "/* Script for -pie -z combreloc -z separate-code -z relro */"
@@ -574,10 +730,20 @@ if test -n "$GENERATE_PIE_SCRIPT"; then
 	) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xdceo
 	rm -f ${COMBRELOC}
 
+	LD_FLAG=rowpietextonly
+	COMBRELOC=ldscripts/${EMULATION_NAME}.xdceor.tmp
+	( echo "/* Script for -pie -z combreloc -z separate-code -z relro -z one-rosegment */"
+	  source_sh ${CUSTOMIZER_SCRIPT}
+	  source_sh ${srcdir}/scripttempl/${SCRIPT_NAME}.sc
+	) | sed -e '/^ *$/d;s/[	 ]*$//' > ldscripts/${EMULATION_NAME}.xdceor
+	rm -f ${COMBRELOC}
+
 	unset RELRO
     fi
+
     COMBRELOC=
   fi
+
   unset CREATE_PIE
 fi
 
